﻿using System;
using System.Configuration;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.ServiceModel;
using System.Threading;
using Antiplagiat.ApiCorp.Model;

namespace ApiCorpExample
{
    class ExamplesNative
    {
        /// <summary>
        /// Адрес сайта "Антиплагиат".
        /// </summary>
        public static Uri AntiplagiatUri { get; private set; }

        /// <summary>
        /// Уникальное имя компании.
        /// </summary>
        public static string CompanyName { get; private set; }

        /// <summary>
        /// Адрес сервиса ApiCorp.
        /// </summary>
        public static string ApiCorpAddress { get; private set; }

        // Система запоминает логин/пароль, который использовался при первом обращении. Указанный при первом обращении логин/пароль используется для дальнейшей верификации запросов.
        public static string Login { get; private set; }
        public static string Password { get; private set; }

        static ExamplesNative()
        {
            AntiplagiatUri = new Uri(ConfigurationManager.AppSettings["antipalgiat.url"]);
            CompanyName = ConfigurationManager.AppSettings["company"];
            ApiCorpAddress = ConfigurationManager.AppSettings["apicorp.addr"];
            Login = ConfigurationManager.AppSettings["login"];
            Password = ConfigurationManager.AppSettings["password"];
        }

        /// <summary>
        /// Подготовка описания загружаемого файла.
        /// </summary>
        /// <param name="fileName">Полный путь до файла.</param>
        /// <returns>Подготовленный объект с описанием и самим загруженным файлом.</returns>
        private static DocData GetDocData(string fileName)
        {
            return new DocData
            {
                Data = File.ReadAllBytes(fileName),
                FileType = Path.GetExtension(fileName),
                FileName = Path.GetFileName(fileName),
                ExternalUserID = "ivanov", //поле надо заполнять идентификатором реального пользователя во внешней системе
            };
        }

        /// <summary>
        /// Пример самого распространенного сценария проверки документа.
        /// </summary>
        /// <param name="fileName">Путь до файла который надо проверять.</param>
        public static void SimpleCheck(string fileName)
        {
            Console.WriteLine("SimpleCheck filename=" + fileName);
            // Создать клиента сервиса
            var client = new Antiplagiat.ApiCorp.ApiCorpClient(CompanyName, ApiCorpAddress);

            // Описание загружаемого файла
            var data = GetDocData(fileName);
            var attributes = new DocAttributes
            {
                DocumentDescription = new VerificationReportOptions()
                {
                    Authors = new[]
                    {
                        new AuthorName
                        {
                            OtherNames = "Иван Иванович",
                            Surname = "Иванов",
                            PersonIDs = new PersonIDs { CustomID = "original" }
                        }
                    }
                }
            };

            // Загрузка файла
            var uploadResult = client.UploadDocument(data, attributes);

            // Идентификатор документа. Если загружается не архив, то список загруженных документов будет состоять из одного элемента.
            var id = uploadResult.Uploaded.First().Id;

            // Отправить на проверку с использованием всех подключеных компании модулей поиска
            client.CheckDocument(id);
            // Отправить на проверку с использованием только собственного модуля поиска и модуля поиска "wikipedia". Для получения списка модулей поиска см. пример GetTariffInfo()
            //client.CheckDocument(id, new[] { CompanyName, "checker" });

            // Получить текущий статус последней проверки
            var status = client.GetCheckStatus(id);

            // Цикл ожидания окончания проверки
            while (ReportStatus.InProgress == status.Status)
            {
                Thread.Sleep(TimeSpan.FromSeconds(status.EstimatedWaitTime.Value));
                status = client.GetCheckStatus(id);
            }

            // Проверка закончилась не удачно.
            if (ReportStatus.Failed == status.Status)
            {
                Trace.TraceInformation("При проверке документа {0} произошла ошибка: {1}", fileName, status.FailDetails);
                return;
            }

            // Получить краткий отчет
            var report = client.GetReportView(id);
            //Cуммарный процент найденных заимствований
            Console.WriteLine("Report Summary: " + (100 - report.Summary.DetailedScore.Unknown) + "%");
            // Детализация процента неоригинального текста с разбивкой на заимствования, легальное цитирование и самоцитирование
            Console.WriteLine("Report BaseScore.Legal: " + report.Summary.BaseScore.Legal + "%");
            Console.WriteLine("Report BaseScore.Plagiarism: " + report.Summary.BaseScore.Plagiarism + "%");
            Console.WriteLine("Report BaseScore.SelfCite: " + report.Summary.BaseScore.SelfCite + "%");
            // Детализация процента неоригинального текста с разбивкой на заимствования, легальное цитирование и самоцитирование после корректировки
            Console.WriteLine("Report DetailedScore.Legal: " + report.Summary.DetailedScore.Legal + "%");
            Console.WriteLine("Report DetailedScore.Plagiarism: " + report.Summary.DetailedScore.Plagiarism + "%");
            Console.WriteLine("Report DetailedScore.SelfCite: " + report.Summary.DetailedScore.SelfCite + "%");

            foreach (var checkService in report.CheckServiceResults)
            {
                // Информация по каждому поисковому модулю
                Console.WriteLine("Check service: {0} ({1}) \n ScoreByReport={2}% \n Score.White={3}% Score.Black={4}% Score.SelfCite={5}% \n"
                    , checkService.CheckServiceName
                    , checkService.CollectionDescription
                    , checkService.ScoreByReport
                    , checkService.ScoreByReport.Legal
                    , checkService.ScoreByReport.Plagiarism
                    , checkService.ScoreByReport.SelfCite
                    );
                if (checkService.Sources == null) continue;
                foreach (var source in checkService.Sources)
                    // Информация по каждому найденному источнику
                    Console.WriteLine("\t{0}: Score={1}%({2}%), Name='{3}' Author='{4}' Url='{5}'",
                        source.SrcHash,
                        source.ScoreByReport, source.ScoreBySource,
                        source.Name, source.Author, source.Url);
            }

            // Получить полный отчет
            var fullReport = client.GetReportView(id, new ReportViewOptions { FullReport = true, 
                NeedText = true, NeedAttributes = true});

            if (fullReport.Details.CiteBlocks != null)
            {
                // Найти самый большой блок заимствований и вывести его
                CiteBlock maxBlock = null;
                foreach (var block in fullReport.Details.CiteBlocks)
                {
                    if ((maxBlock == null) || block.Length > maxBlock.Length)
                    {
                        maxBlock = block;
                    }
                }
                if (maxBlock != null)
                    Console.WriteLine("Max block length={0} Source={1} text:\n{2}...", maxBlock.Length, maxBlock.SrcHash,
                        fullReport.Details.Text.Substring(maxBlock.Offset, Math.Min(maxBlock.Length, 200)));
            }

            var authorsData = fullReport.Attributes.DocumentDescription.Authors;
            Console.WriteLine("Author Surname={0} OtherNames={1} CustomId={2}",
                authorsData[0].Surname, authorsData[0].OtherNames, authorsData[0].PersonIDs.CustomID);
        }

        /// <summary>
        /// Пример демонстрирующий как корректно вести себя при ошибках во время загрузки документа на првоерку.
        /// </summary>
        /// <param name="fileName">Путь до файла который надо проверять.</param>
		public static void IncorrectUpload(string fileName)
        {
            Console.WriteLine("IncorrectUpload filename=" + fileName);
            // Создать клиента сервиса
            var client = new Antiplagiat.ApiCorp.ApiCorpClient(CompanyName, ApiCorpAddress, Login, Password).UseMtomMessageEnconding();

            // Описание загружаемого файла
            var data = GetDocData(fileName);

            // Загрузка файла
            try
            {
                var uploadResult = client.UploadDocument(data);
                Console.WriteLine("Файл успешно загружен");
            }
            catch (FaultException<InvalidArgumentException> e)
            {
                // Данный файл не может быть загружен. Конкретная причина указана в поле Message.
                // Повторять вызов с теми же параметрами бессмыслено, результат будет тот же.
                Console.WriteLine("Файл не может быть загружен: " + e.Message);
            }
            catch (FaultException e)
            {
                // Проблема не связана с конкретным файлом имеет смысл повторить попытку позже
                Console.WriteLine("При загрузке файла произошла ошибка: " + e.Message);
            }
        }


        /// <summary>
        /// Перебор всех полученных отчетов
        /// </summary>
        public static void EnumerateReports()
        {
            Console.WriteLine("EnumerateReports");
            // Создать клиента сервиса
            var client = new Antiplagiat.ApiCorp.ApiCorpClient(CompanyName, ApiCorpAddress, Login, Password).UseMtomMessageEnconding();
            // выбираем документы пользователя с идентификатором во внешней системе "ivanov"
            var opts = new EnumerateReportsOptions { ExternalUserID = "ivanov", Count = 10 };
            // Получить первую пачку отчетов, сортировка всегда обратная хронологическая, отчет последнего загруженного документа будет первым
            var reps = client.EnumerateReportInfos(opts);
            // Счетчик отчетов
            var i = 0;
            // Функция EnumerateReportInfos возвращает результаты проверки пачками. Пустая пачка означает, что перебор завершился.
            while (reps.Length > 0)
            {
                foreach (var info in reps)
                {
                    ++i;
                    // Выводим номер по порядку, название документа и статус последнего отчета
                    Console.WriteLine("{0}\tDocName: {1}\n   Status: {2}", i, info.DocumentInfo.Attributes.Name, info.CheckStatus.Status);
                    // Если проверка завершена, то выводим процет неоригинальности и ссылку на отчет.
                    if (info.CheckStatus.Status == ReportStatus.Ready)
                        Console.WriteLine("{0}, {1}", 100 - info.CheckStatus.Summary.DetailedScore.Unknown, new Uri(AntiplagiatUri, info.CheckStatus.Summary.ReportWebId));
                }
                // Получить следующую пачку отчетов
                opts.Skip = i;
                reps = client.EnumerateReportInfos(opts);
            }
        }


        /// <summary>
        /// Перебор всех загруженных документов
        /// </summary>
        public static void EnumerateDocuments()
        {
            Console.WriteLine("EnumerateDocuments");
            // Создать клиента сервиса
            var client = new Antiplagiat.ApiCorp.ApiCorpClient(CompanyName, ApiCorpAddress, Login, Password).UseMtomMessageEnconding();
            // Получить первую пачку документов
            var docs = client.EnumerateDocuments(null);
            // Счетчик документов
            var i = 0;
            // Функция EnumerateDocuments возвращает идентификаторы пачками. Пустая пачка означает, что перебор завершился.
            while (docs.Length > 0)
            {
                foreach (var id in docs)
                {
                    ++i;
                    // Каждый пятый документ добавляем в индекс
                    if ((i % 5) == 0)
                    {
                        client.SetIndexState(id, IndexState.Indexed);
                    }

                    // Каждому третьему добавляем атрибут
                    if ((i % 3) == 0)
                    {
                        var docnum = new CustomAttribute { AttrName = "docnum", AttrValue = "" + i };
                        client.UpdateDocumentAttributes(id, new DocAttributes { Custom = new[] { docnum } });
                    }
                }
                // Получить следующую пачку идентификаторов документов
                docs = client.EnumerateDocuments(docs[docs.Length - 1]);
            }
            Console.WriteLine("Total documents count: {0} ({1})", i, client.GetCompanyStats().TotalCount);
        }

        /// <summary>
        /// Перебор всех документов добавленных в индекс
        /// </summary>
        public static void EnumerateIndex()
        {
            Console.WriteLine("EnumerateIndex");
            // Создать клиента сервиса
            var client = new Antiplagiat.ApiCorp.ApiCorpClient(CompanyName, ApiCorpAddress, Login, Password).UseMtomMessageEnconding();

            // При вызове EnumerateDocuments в опциях указывается, что требуются только документы добавленные в индекс
            var enumerateOptions = new EnumerateDocumentsOptions { AddedToIndex = true };
            var docs = client.EnumerateDocuments(null, enumerateOptions);
            // Счетчик документов
            var i = 0;
            // Функция EnumerateDocuments возвращает идентификаторы пачками. Пустая пачка означает, что перебор завершился.
            while (docs.Length > 0)
            {
                foreach (var id in docs)
                {
                    ++i;
                    // Каждый второй документ удаляем из индекса
                    if ((i % 2) == 0)
                    {
                        client.SetIndexState(id, IndexState.None);
                    }
                }
                // Получить следующую пачку идентификаторов документов
                docs = client.EnumerateDocuments(docs[docs.Length - 1], enumerateOptions);
            }
            Console.WriteLine("Total indexed documents count: {0} ({1})", i, client.GetCompanyStats().AddedToIndexCount);
        }

        /// <summary>
        /// Проверить документ, получить различные ссылки на отчет на сайте "Антиплагиат"
        /// </summary>
        /// <param name="fileName">Путь до файла который надо проверять.</param>
        public static void GetWebReport(string fileName)
        {
            Console.WriteLine("GetWebReport filename=" + fileName);
            // Создать клиента сервиса
            var client = new Antiplagiat.ApiCorp.ApiCorpClient(CompanyName, ApiCorpAddress, Login, Password).UseMtomMessageEnconding();

            // Описание загружаемого файла
            var data = GetDocData(fileName);

            // Загрузка файла
            var uploadResult = client.UploadDocument(data);

            // Идентификатор документа. Если загружается не архив, то список загруженных документов будет состоять из одного элемента.
            DocumentId id = uploadResult.Uploaded.First().Id;

            // Отправить на проверку
            Console.WriteLine(id.ToString());
            client.CheckDocument(id);

            // Получить текущий статус последней проверки
            var status = client.GetCheckStatus(id);

            // Цикл ожидания окончания проверки
            while (ReportStatus.InProgress == status.Status)
            {
                var estTime = TimeSpan.FromSeconds(status.EstimatedWaitTime.Value);
                Console.WriteLine(estTime);
                Thread.Sleep(estTime);
                status = client.GetCheckStatus(id);
            }

            // Проверка закончилась не удачно.
            if (ReportStatus.Failed == status.Status)
            {
                Trace.TraceInformation("При проверке документа {0} произошла ошибка: {1}", fileName, status.FailDetails);
                return;
            }

            // Получить ссылку на полный отчет на сайте "Антиплагиат"
            var webFullUrl = new Uri(AntiplagiatUri, status.Summary.ReportWebId);
            Console.WriteLine("Full Report: " + webFullUrl);

            // Получить ссылку на краткий отчет на сайте "Антиплагиат"
            var webShortUrl = new Uri(AntiplagiatUri, status.Summary.ShortReportWebId);
            Console.WriteLine("Short Report: " + webShortUrl);

            // Получить ссылку на отчет на сайте "Антиплагиат"
            var webReadonlyUrl = new Uri(AntiplagiatUri, status.Summary.ReadonlyReportWebId);
            Console.WriteLine("Readonly Report: " + webReadonlyUrl);
        }

        /// <summary>
        /// Пример демонстрирующий методы для работы с папками.
        /// </summary>
        /// <param name="fileName1">Путь до первого файла который будет загружаться в систему.</param>
        /// <param name="fileName2">Путь до второго файла который будет загружаться в систему.</param>
        public static void FoldersMethods(string fileName1, string fileName2)
        {
            Console.WriteLine("FoldersMethods");
            // Создать клиента сервиса
            var client = new Antiplagiat.ApiCorp.ApiCorpClient(CompanyName, ApiCorpAddress, Login, Password).UseMtomMessageEnconding();

            const string email = "testapi@antiplagiat.ru";

            // Пользователь создаёт папку "New Folder"
            var folderId = client.AddFolder(email, "New folder");

            // Пользователь переименовывает её в "Documents"
            client.RenameFolder(email, folderId, "Documents");

            // Опции создания папки.
            var folderOptions = new FolderOptions { ParentId = folderId };

            // Пользователь создаёт в папке "Documents" новую папку "Important"
            var importantFolderId = client.AddFolder(email, "Important", folderOptions);

            // Описание первого загружаемого файла
            var data1 = GetDocData(fileName1);

            // Пользователь загружает в папку "Documents" файл fileName1
            client.UploadDocument(data1, null, new UploadOptions { FolderId = folderId, FromUser = email });

            // Описание второго загружаемого файла
            var data2 = GetDocData(fileName2);

            // Пользователь загружает в папку "Important" файл fileName2
            client.UploadDocument(data2, null, new UploadOptions { FolderId = importantFolderId, FromUser = email });

            // Пользователь создаёт папку "Trash"
            var trashFolderId = client.AddFolder(email, "Trash");

            // Пользователь получает идентификаторы всех своих документов
            var docIds = client.EnumerateDocuments(null, new EnumerateDocumentsOptions { FromUser = email });

            // Пользователь перемещает свои документы в папку "Trash"
            client.MoveDocuments(email, trashFolderId, docIds);

            // Пользователь получает дерево папок
            var folderTree = client.GetFolders(email);

            Console.WriteLine("FolderTree:");

            Console.WriteLine("\nTop level:");
            Console.WriteLine(string.Format("Name: {0}", folderTree.Name));

            Console.WriteLine("\nNext level:");

            foreach (var folder in folderTree.Children)
                Console.WriteLine(string.Format("Name: {0}", folder.Name));

            Console.WriteLine("\nGet documents from \"Trash\"");

            // Пользователь получает свои документы из папки "Trash"
            var trashDocs = client.EnumerateDocuments(null, new EnumerateDocumentsOptions { InStorage = false, FolderId = trashFolderId, FromUser = email });

            Console.WriteLine("Documents count: {0}", trashDocs.Length);

            // Пользователь удаляет папки "Documents" и "Trash"
            client.DeleteFolder(email, folderId);
            client.DeleteFolder(email, trashFolderId);
        }

        /// <summary>
        /// Методы для работы с хранилищем.
        /// </summary>
        /// <param name="fileName1">Путь до первого файла который будет загружаться в систему.</param>
        /// <param name="fileName2">Путь до второго файла который будет загружаться в систему.</param>
        public static void StorageMethods(string fileName1, string fileName2)
        {
            Console.WriteLine("StorageMethods");
            // Создать клиента сервиса
            var client = new Antiplagiat.ApiCorp.ApiCorpClient(CompanyName, ApiCorpAddress, Login, Password).UseMtomMessageEnconding();

            // Пользователь создаёт папку "New Folder" в хранилище
            var folderId = client.AddFolder(null, "New folder");

            // Пользователь переименовывает её в "Documents"
            client.RenameFolder(null, folderId, "Documents");

            // Опции создания папки.
            var folderOptions = new FolderOptions { ParentId = folderId };

            // Пользователь создаёт в папке "Documents" новую папку "Important"
            var importantFolderId = client.AddFolder(null, "Important", folderOptions);

            // Описание первого загружаемого файла
            var data1 = GetDocData(fileName1);

            // Пользователь загружает в папку "Documents" файл fileName1
            client.UploadDocument(data1, null, new UploadOptions { FolderId = folderId, ToStorage = true });

            // Описание второго загружаемого файла
            var data2 = GetDocData(fileName2);

            // Пользователь загружает в папку "Important" файл fileName2
            client.UploadDocument(data2, null, new UploadOptions { FolderId = importantFolderId, ToStorage = true });

            // Пользователь создаёт папку "Trash" в хранилище
            var trashFolderId = client.AddFolder(null, "Trash");

            // Пользователь получает идентификаторы всех документов из хранилища
            var docIds = client.EnumerateDocuments(null);

            // Пользователь перемещает все документы в папку "Trash"
            client.MoveDocuments(null, trashFolderId, docIds);

            // Пользователь получает дерево папок хранилища
            var folderTree = client.GetFolders(null);

            Console.WriteLine("FolderTree:");

            Console.WriteLine("\nTop level:");
            Console.WriteLine(string.Format("Name: {0}", folderTree.Name));

            Console.WriteLine("\nNext level:");

            foreach (var folder in folderTree.Children)
                Console.WriteLine(string.Format("Name: {0}", folder.Name));

            Console.WriteLine("\nGet documents from \"Trash\"");

            // Пользователь получает документы хранилища из папки "Trash"
            var trashDocs = client.EnumerateDocuments(null, new EnumerateDocumentsOptions { InStorage = true, FolderId = trashFolderId });

            Console.WriteLine("Documents count: {0}", trashDocs.Length);

            // Пользователь удаляет папки "Documents" и "Trash"
            client.DeleteFolder(null, folderId);
            client.DeleteFolder(null, trashFolderId);
        }

        /// <summary>
        /// Демонстрация методов получения информации о текущем тарифе.
        /// </summary>
        public static void GetTariffInfo()
        {
            Console.WriteLine("GetTariffInfo");

            // Создать клиента сервиса
            var client = new Antiplagiat.ApiCorp.ApiCorpClient(CompanyName, ApiCorpAddress, Login, Password).UseMtomMessageEnconding();

            // Получить информацию о текущем тарифе
            var tariffInfo = client.GetTariffInfo();

            Console.WriteLine(string.Format("Наименование тарифа: {0}", tariffInfo.Name));
            Console.WriteLine(string.Format("Дата подключения тарифа: {0}", tariffInfo.SubscriptionDate));
            Console.WriteLine(string.Format("Дата окончания тарифа: {0}", tariffInfo.ExpirationDate));
            Console.WriteLine(string.Format("Исходное количество проверок: {0}", tariffInfo.TotalChecksCount));
            Console.WriteLine(string.Format("Оставшееся количество проверок: {0}", tariffInfo.RemainedChecksCount));

            Console.WriteLine(string.Format("\nСписок проверяющих сервисов, по которым возможно осуществлять проверки:"));

            foreach (var checkServiceInfo in tariffInfo.CheckServices)
                Console.WriteLine(string.Format("{0} ({1})", checkServiceInfo.Code, checkServiceInfo.Description));
        }

        /// <summary>
        /// Пример демонстрирующий получение pdf-отчета.
        /// </summary>
        /// <param name="fileName">Путь до файла который будет проверяться.</param>
        public static void ExportReportToPdf(string fileName)
        {
            Console.WriteLine("ExportReportToPdf filename=" + fileName);
            // Создать клиента сервиса
            var client = new Antiplagiat.ApiCorp.ApiCorpClient(CompanyName, ApiCorpAddress, Login, Password).UseMtomMessageEnconding();

            // Описание загружаемого файла
            var data = GetDocData(fileName);

            // Загрузка файла
            var uploadResult = client.UploadDocument(data);

            // Идентификатор документа. Если загружается не архив, то список загруженных документов будет состоять из одного элемента.
            DocumentId id = uploadResult.Uploaded.First().Id;

            // Отправить на проверку
            client.CheckDocument(id);

            // Получить текущий статус последней проверки
            var status = client.GetCheckStatus(id);

            // Цикл ожидания окончания проверки
            while (ReportStatus.InProgress == status.Status)
            {
                Thread.Sleep(TimeSpan.FromSeconds(status.EstimatedWaitTime.Value));
                status = client.GetCheckStatus(id);
            }

            // Проверка закончилась неудачно.
            if (ReportStatus.Failed == status.Status)
            {
                Trace.TraceInformation("При проверке документа {0} произошла ошибка: {1}", fileName, status.FailDetails);
                return;
            }

            // Запросить формирование последнего полного отчета в формат PDF.
            var exportReportInfo = client.ExportReportToPdf(id);

            // Цикл ожидания формирования отчета
            while (ExportReportStatus.InProgress == exportReportInfo.Status)
            {
                Thread.Sleep(TimeSpan.FromSeconds(exportReportInfo.EstimatedWaitTime.Value));
                exportReportInfo = client.ExportReportToPdf(id);
            }

            // Формирование отчета закончилось неудачно.
            if (ExportReportStatus.Failed == exportReportInfo.Status)
            {
                Trace.TraceInformation("При формировании PDF-отчета для документа {0} произошла ошибка.", fileName);
                return;
            }

            // Получить ссылку на отчет на сайте "Антиплагиат"
            // ВНИМАНИЕ! Не гарантируется что данная ссылка будет работать вечно, она может перестать работать в любой момент,
            // поэтому нельзя давать ее пользвателю. Нужно скачивать pdf себе и дальше уже управлять его временем жизни
            var downloadLink = new Uri(AntiplagiatUri, exportReportInfo.DownloadLink);
            Console.WriteLine($"PDF full report (number = {exportReportInfo.ReportNum}): " + downloadLink);

            // Опции для формирования краткого отчета с номером 1 в формат PDF.
            var options = new ExportReportOptions
            {
                ReportNum = 1,
                ShortReport = true
            };

            // Запросить формирование отчета в формат PDF с указанными опциями.
            exportReportInfo = client.ExportReportToPdf(id, options);

            // Цикл ожидания формирования отчета
            while (ExportReportStatus.InProgress == exportReportInfo.Status)
            {
                Thread.Sleep(TimeSpan.FromSeconds(exportReportInfo.EstimatedWaitTime.Value));
                exportReportInfo = client.ExportReportToPdf(id, options);
            }

            // Формирование отчета закончилось неудачно.
            if (ExportReportStatus.Failed == exportReportInfo.Status)
            {
                Trace.TraceInformation("При формировании PDF-отчета для документа {0} произошла ошибка.", fileName);
                return;
            }

            // Получить ссылку на отчет на сайте "Антиплагиат"
            // ВНИМАНИЕ! Не гарантируется что данная ссылка будет работать вечно, она может перестать работать в любой момент,
            // поэтому нельзя давать ее пользвателю. Нужно скачивать pdf себе и дальше уже управлять его временем жизни.
            downloadLink = new Uri(AntiplagiatUri, exportReportInfo.DownloadLink);
            Console.WriteLine($"PDF short report (number = {exportReportInfo.ReportNum}): " + downloadLink);
        }

        /// <summary>
        /// Пример получения справки по проверенному файлу
        /// </summary>
        /// <param name="fileName">Путь до файла который будет проверяться</param>
        public static void GetVerificationReport(string fileName)
        {
            Console.WriteLine("GetVerificationReport fileName={0}", fileName);

            // Создать клиента сервиса
            var client = new Antiplagiat.ApiCorp.ApiCorpClient(CompanyName, ApiCorpAddress, Login, Password).UseMtomMessageEnconding();

            // Описание загружаемого файла
            var data = GetDocData(fileName);

            // Загрузка файла
            var uploadResult = client.UploadDocument(data);

            // Идентификатор документа. Если загружается не архив, то список загруженных документов будет состоять из одного элемента.
            DocumentId id = uploadResult.Uploaded.First().Id;

            // Отправить на проверку
            Console.WriteLine("Проверка документа: {0}", id.ToString());
            client.CheckDocument(id);

            // Получить текущий статус последней проверки
            var status = client.GetCheckStatus(id);

            // Цикл ожидания окончания проверки
            while (ReportStatus.InProgress == status.Status)
            {
                var estTime = TimeSpan.FromSeconds(status.EstimatedWaitTime.Value);
                Console.WriteLine("Ожидаемое время обработки: {0}", estTime);
                Thread.Sleep(estTime);
                status = client.GetCheckStatus(id);
            }

            // Проверка закончилась неудачно.
            if (ReportStatus.Failed == status.Status)
            {
                Trace.TraceInformation("При проверке документа {0} произошла ошибка: {1}", fileName, status.FailDetails);
                return;
            }

            try
            {
                //Выгрузить справку можно c заполненными полями или без
                //Без заполненных полей
                //var reportBytes = client.GetVerificationReport(id);

                //Для заполнения полей необходимо создать объект опций справки с желаемыми значениями свойств
                var options = new VerificationReportOptions
                {
                    Author = "Иванов И.И.", // ФИО автора работы
                    Department = "Факультет общей и прикладной физики", // Факультет (структурное подразделение)
                    ShortReport = true, // Требуется ли ссылка на краткий отчёт? (qr код)
                    Type = "Дипломная работа", // Тип работы
                    Verifier = "Петров П.П.", // ФИО проверяющего
                    Work = "Когерентная спиновая динамика двумерного электронного газа"  // Название работы
                };

                var reportPath = getVerReportName(id.Id, options);

                //С заполнением полей
                var reportBytesWithFields = client.GetVerificationReport(id, options);

                //Запись в файл
                File.WriteAllBytes(reportPath, reportBytesWithFields);
            }
            catch (FaultException<InvalidArgumentException> e)
            {
                // У документа нет отчёта/закрытого отчёта или в качестве id в GetVerificationReport передан null
                Console.WriteLine("Справка для документа не может быть получена: " + e.Message);
            }
            catch (FaultException<DocumentIdException> e)
            {
                // Указан невалидный DocumentId, когда передан null бросается FaultException<InvalidArgumentException>
                Console.WriteLine("Справка для документа не может быть получена из-за неверно переданного DocumentId: " + e.Message);
            }
            catch (FaultException e)
            {
                // Проблема не связана с передаваемыми параметрами, имеет смысл повторить попытку позже
                Console.WriteLine("Справка для документа не может быть получена: " + e.Message);
            }
        }

        /// <summary>
        /// Проверка расчета самоцитирования по собственной коллекции.
        /// </summary>
        /// <param name="fileName">Путь до файла который надо проверять.</param>
        public static void SelfCiteCheck(string fileName)
        {
            Console.WriteLine("SelfCite filename=" + fileName);
            // Создать клиента сервиса
            var client = new Antiplagiat.ApiCorp.ApiCorpClient(CompanyName, ApiCorpAddress);

            // Описание загружаемого файла
            var data = GetDocData(fileName);
            var attributes = new DocAttributes
            {
                DocumentDescription = new VerificationReportOptions()
                {
                    Authors = new[]
                    {
                        new AuthorName
                        {
                            OtherNames = "Иван Иванович",
                            Surname = "Иванов",
                            PersonIDs = new PersonIDs { CustomID = "original" }
                        }
                    }
                }
            };
            var options = new UploadOptions
            {
                AddToIndex = true
            };

            // Загрузка файла
            var uploadResult = client.UploadDocument(data, attributes, options);

            // Идентификатор документа. Если загружается не архив, то список загруженных документов будет состоять из одного элемента.
            var id = uploadResult.Uploaded.First().Id;

            // Отправить на проверку с использованием всех подключеных компании модулей поиска
            client.CheckDocument(id, new[] { CompanyName });

            // Получить текущий статус последней проверки
            var status = client.GetCheckStatus(id);

            // Цикл ожидания окончания проверки
            while (ReportStatus.InProgress == status.Status)
            {
                Thread.Sleep(TimeSpan.FromSeconds(status.EstimatedWaitTime.Value));
                status = client.GetCheckStatus(id);
            }

            // Проверка закончилась не удачно.
            if (ReportStatus.Failed == status.Status)
            {
                Trace.TraceInformation("При проверке документа {0} произошла ошибка: {1}", fileName, status.FailDetails);
                return;
            }

            //грузим тот же файл с тем же Id автора чтобы проверить расчет самоцитирования
            uploadResult = client.UploadDocument(data, attributes);
            var idSelfCite = uploadResult.Uploaded.First().Id;

            client.CheckDocument(idSelfCite, new[] { CompanyName });

            // Получить текущий статус последней проверки
            status = client.GetCheckStatus(idSelfCite);

            // Цикл ожидания окончания проверки
            while (ReportStatus.InProgress == status.Status)
            {
                Thread.Sleep(TimeSpan.FromSeconds(status.EstimatedWaitTime.Value));
                status = client.GetCheckStatus(idSelfCite);
            }

            // Получить краткий отчет
            var report = client.GetReportView(idSelfCite);
            // Должно быть самоцитирование 100%
            Console.WriteLine("Report BaseScore.SelfCite: " + report.Summary.BaseScore.SelfCite + "%");
        }

        private static string getVerReportName(int? id, VerificationReportOptions verReportOptions)
        {
            var docId = id.HasValue ? id.Value + "_" : "";
            string author = "";

            if (verReportOptions != null)
                if (!string.IsNullOrEmpty(verReportOptions.Author))
                    author = "_" + verReportOptions.Author;

            return $"Сertificate_{docId}{DateTime.Now.ToString("yyyyMMdd")}{author}.pdf";
        }
    }
}